#include <stdio.h>
#include <iostream>
#include <stdlib.h>
#include <string.h>
#include <cuda.h>
#include "Chi2.h"

int main(int argc ,char* argv[]) {

FILE *fp;
size_t size;
	  
/* Initialize rows, cols, ncases, ncontrols from the user */
unsigned int rows=atoi(argv[2]);
unsigned int cols=atoi(argv[3]);
int ncases=atoi(argv[4]);
int ncontrols=atoi(argv[5]);
float bcutoff = atof(argv[6]);
int CUDA_DEVICE = atoi(argv[7]);
int THREADS = atoi(argv[8]);
printf("Individuals=%d SNPs=%d cases=%d controls=%d Bonferroni-cutoff=%f CUDA_DEVICE=%d\n",rows,cols,ncases,ncontrols,bcutoff,CUDA_DEVICE);

cudaError err = cudaSetDevice(CUDA_DEVICE);
if(err != cudaSuccess) { printf("Error setting CUDA DEVICE\n"); exit(EXIT_FAILURE); }

/*Host variable declaration */

//int THREADS = 32;
int BLOCKS;
float host_results[cols];
unsigned int jobs; 
int transfer = 0;
unsigned long ulone = 1;
unsigned long ultwo = 2; 

sclHard* hardware = sclGetAllHardware(&found);
sclPrintHardwareStatus(*hardware);

/*Kernel variable declaration */
unsigned char *dev_dataT;
float *results;

/* Variable initialized to perform univariate tests */

/* set the cut off */

float cutoff = 0.05/cols;
printf("Univariate cutoff is %e\n",cutoff);

/* Validation to check if the data file is readable */
fp = fopen(argv[1], "r");
if (fp == NULL) {
    	printf("Cannot Open the File");
	return 0;
}

size = (size_t)(rows * cols);
printf("Size of the data = %u\n",size);

fflush(stdout);

unsigned char *dataT = (unsigned char*)malloc((size_t)size);

if(dataT == NULL) {
        printf("ERROR: Memory for data not allocated.\n");
}

/* Transfer the SNP Data from the file to CPU Memory */
for(unsigned int i=0 ; i < size; i++ ) {
	int tmp;
	fscanf(fp,"%d",&tmp);
	dataT[i] = (char)(((int)'0')+tmp);
	if(i == size - 1){ 
		transfer = 1; 
		printf("SNP Data Transferred to the Memory....\nProcessing\n");
   	}
} 
fclose(fp);

if(transfer != 1){
        printf("ERROR: Complete data is not transferred to the Memory.\n");
}

/* allocate the Memory in the GPU for SNP data */	   
cudaMalloc((unsigned char**) &dev_dataT, size * sizeof(unsigned char) );
cudaMalloc((float**) &results, cols * sizeof(float) );

/*Copy the SNP data to GPU */
cudaMemcpy(dev_dataT, dataT, size * sizeof(unsigned char), cudaMemcpyHostToDevice);

jobs = cols;
BLOCKS = (jobs + THREADS - 1)/THREADS;

/*Calling the kernel function */
kernel<<<BLOCKS,THREADS>>>(rows,cols,ncases,ncontrols,dev_dataT,results);
		
/*Copy the results back in host*/
cudaMemcpy(host_results,results,cols * sizeof(float),cudaMemcpyDeviceToHost);
		
for(int k = 0; k < jobs; k++) {
	printf("%f ", host_results[k]);
}
printf("\n");

cudaFree( dev_dataT );
cudaFree( results );
return 0;

}
